% -------------------------------------------------------------------------
% Must Run Calibraton.m before running this script - need to have already
% established a transformation matrix ('Transmatrix.mat')

% Script to test the calibration of the Kinect Camera to the Fanuc Robot
% Moves robot to random locations and locates X in Kinect image.
% The calculated position of the Fanuc, based on the Kinect and Transformation matrix, 
% is then compared to the position that the Fanuc was sent to.
% -------------------------------------------------------------------------
% Launch the teaching pendant program LV_COM_FRM, and run Calibration.m before launching this script.
% See: http://classes.engineering.wustl.edu/ese497/index.php/Fanuc_How_Tos,_FAQ,_etc.#FAQ
% for detailed instructions.
% -------------------------------------------------------------------------
% See comments at top of Matlab2Fanuc for details of the robot communication
% See comments at the top of Kinect2SharedMem2Matlab.m for details of the
% Kinect camera communication.
% -------------------------------------------------------------------------

clear all; close all;
host='localhost';   
load TransMatrix.mat
collectData = 1 ;

% Launch LabVIEW program that receives TCP/IP commands and communicates
% with the Robot
!start /min "" LabVIEW2Fanuc\builds\MatlabLabVIEWFanuc\MatlabLabVIEWFanuc.exe
pause(2) ; % Make sure this is open so we don't have communication errors

h1 = figure(11) ;
h2 = figure(12) ;
h3 = figure(13) ;
h4 = figure(14) ;
h5 = figure(15) ;

kinectCam = Kinect() ;
[ cross, templateX ] = CreateTemplates() ;
rangeXYZ=[[375,525];[-150,150];[-250,-150]] ;
numPoints = 10 ;
searchWinX = [130,510] ;
searchWinY = [100,410] ;
frame = 1 ;

for i = 1:numPoints
    
    x = rangeXYZ(1,1)+rand*(rangeXYZ(1,2) - rangeXYZ(1,1))  ;
    y = rangeXYZ(2,1)+rand*(rangeXYZ(2,2) - rangeXYZ(2,1))  ;
    z = rangeXYZ(3,1)+rand*(rangeXYZ(3,2) - rangeXYZ(3,1))  ;
   
            fanuc(frame,1) = x ;
            fanuc(frame,2) = y ;
            fanuc(frame,3) = z ;
            if ( collectData )
                [data_received]=exchangeData(host,Double2CSV([x y z -90 -90 -90 0 0]));
                               
                pause(1) ;
                [rgb(:,:,:,frame),depth(:,:,frame)] = kinectCam.GetImage() ;
            end
            figure(h1) ; h1imagesc = imagesc(depth(:,:,frame)) ;
            caxis([350 1000]) ;
            figure(h2) ; h2image = image(rgb(:,:,:,frame)) ;

            depthWin = depth(searchWinY(1):searchWinY(2),searchWinX(1):searchWinX(2),frame) ;
            rgbWin = rgb(searchWinY(1):searchWinY(2),searchWinX(1):searchWinX(2),:,frame) ;

            template = templateX ;
            c1 = normxcorr2(template,rgb2gray(rgbWin)) ;
            c = c1((size(template,1)-1)/2+1:(size(template,1)-1)/2+size(rgbWin,1),(size(template,2)-1)/2+1:(size(template,2)-1)/2+size(rgbWin,2)) ;
            figure(h3) ; imagesc(c) ;
            highCCIndices = find(c > 0.5) ;

            [maxCC, indxMaxCC] = max(c(highCCIndices)) ;
            target(frame,1) = floor((highCCIndices(indxMaxCC)-1)/size(rgbWin,1)) + 1 + searchWinX(1) -1 ;
            target(frame,2) = mod((highCCIndices(indxMaxCC)-1),size(rgbWin,1))+ 1 + searchWinY(1) -1 ;
            medianDepth = DepthFilter(depth(target(frame,2)-20:target(frame,2)+20, ...
            target(frame,1)-20:target(frame,1)+20,frame),3,10) ;
            target(frame,3) = medianDepth(21,21) ;
            figure(h2) ; hold on ; plot(target(frame,1),target(frame,2),'rx') ; hold off ;
            drawnow
            frame = frame + 1 ;
      
end
% See FindBestFOVs.m to measure best FOV for this Kinect
cart_mm = proj2cart(target,size(depth,2),size(depth,1),63.1,49.6) ;
% figure ;plot3(cart_mm(:,1),cart_mm(:,2),cart_mm(:,3),'bx') ; xlabel('x') ; ylabel('y') ; zlabel('z') ; grid on ;

% Compute error - appending a one into the bottom row of cart_mm
F = T*([cart_mm'; ones(1,size(cart_mm,1))]) ;
F = F(1:3,:) ;
figure(h4) ;
plot3(fanuc(:,1),fanuc(:,2),fanuc(:,3),'bx') ; xlabel('x') ; ylabel('y') ; zlabel('z') ; grid on ;
hold on ;
plot3(F(1,:),F(2,:),F(3,:),'go') ;
hold off ;
legend('Ideal','Transformed') ;
error = F - fanuc' ;
figure(h5) ; plot(sqrt(diag(error'*error)),'rx')
% Shutdown Kinect and Shared Memory
if ( collectData )
    kinectCam.ShutDown() ;
    % Kill (ungracefully) LabVIEW communication program
    !taskkill /IM MatlabLabVIEWFanuc.exe
end
